#ifndef DEF_MATRIX
#define DEF_MATRIX
#include <assert.h>
#include "vector.h"
//#define DEBUG_MATRIX


struct matrix
/** This file declares struct matrix, which implements
    %matrix with %elements type of double.
    There are also declarations of the functions for the %matrix and %vector computing */
{
  long m;    ///< number of rows
  long n;    ///< number of columns
  double *a; ///< pointer to onedimensional array with matrix elements stored in the rows
  
  matrix() {m = n = 0; a = NULL;}; ///< default constructor
  matrix(long m, long n);          ///< allocating constructor
  matrix(const matrix &mat);       ///< copy constructor
  
  /**
   The operator enables access to the rows of elements of the member array a.

    @param i is the number of row

   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double* operator [] (long i) const
  {
    #ifdef DEBUG_MATRIX
     if (i >= m)
       print_err("matrix row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, m-1);

     assert(i < m);
    #endif
    return (&a[i * n]);
  };


  /**
   The operator enables access to the elements of the member array a.

    @param i is the index of row
    @param j is the index of column

   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double& operator () (long i, long j) const
  {
    #ifdef DEBUG_MATRIX
     if (i >= m)
       print_err("matrix row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, m-1);
     if (j >= n)
       print_err("matrix column index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, j, n-1);

     assert((i < m) && (j < n));
    #endif
    return (a[i * n + j]);
  };

  ~matrix(); ///< destructor
};


struct imatrix
/** This file declares struct imatrix, which implements
    %matrix with %elements type of long. */
{
  long m;    ///< number of rows
  long n;    ///< number of columns
  long *a;   ///< pointer to onedimensional array with matrix elements stored in the rows
  
  imatrix() {m = n = 0; a = NULL;}; ///< default constructor
  imatrix(long m, long n);          ///< allocating constructor
  imatrix(const imatrix &mat);      ///< copy constructor
  
  /**
   The operator enables access to the rows of elements of the member array a.

    @param i is the number of row

   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long* operator [] (long i)
  {
    #ifdef DEBUG_MATRIX
     if (i >= m)
       print_err("imatrix row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, m-1);

     assert(i < m);
    #endif
    return (&a[i * n]);
  };

  /**
   The operator enables access to the elements of the member array a.

    @param i is the index of row
    @param j is the index of column

   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator () (long i, long j)
  {
    #ifdef DEBUG_MATRIX
     if (i >= m)
       print_err("imatrix row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, m-1);
     if (j >= n)
       print_err("imatrix column index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, j, n-1);

     assert((i < m) && (j < n));
    #endif
    return (a[i * n + j]);
  };

  ~imatrix(); ///< destructor
};


/// allocates matrix
long allocm(long m, long n, matrix &mat);
/// allocates imatrix
long allocm(long m, long n, imatrix &mat);
/// allocates double array
long allocm(long m, long n, double **&mat);

/// allocates matrix
long reallocm(long m, long n, matrix &mat);
/// allocates imatrix
long reallocm(long m, long n, imatrix &mat);

/// copies contents of matrix
long copym(const matrix &src, matrix &dest);

/// fills contents of matrix with given value
long fillm(double c, matrix &mat);
long fillrow(double c, long i, matrix &mat);
long fillcol(double c, long i, matrix &mat);

/// deallocates matrix
long destrm(matrix &mat);
long destrm(double **&mat, long m);
long destrm(long **&mat, long m);

/// adds 2 matrices
long addm(const matrix &a, const matrix &b, matrix &c);

/// substracts 2 matrices
long subm(const matrix &a, const matrix &b, matrix &c);

/// performs tensor product of two vectors
void tensprd (vector &a,vector &b,matrix &c);

/// multiplies two matrices A.B=C
long mxm(const matrix &a, const matrix &b, matrix &c);
void mxm(const double *a, const double *b, double *c, long l, long m, long n);

/// multiplies two matrices A.B^T=C
long mxmt(const matrix &a, const matrix &b, matrix &c);
void mxmt(const double *a, const double *b, double *c, long l, long m, long n);

/// multiplies two matrices A^T.B=C
long mtxm(const matrix &a, const matrix &b, matrix &c);
void mtxm(const double *a, const double *b, double *c, long l, long m, long n);
///
void mtxmccr(const double *a, const double *b, double *c, long l, long m, long n);

///
long cmulm(const double c, const matrix &a, matrix &b);
///
long cmulm(double c, matrix &a);

///
long tranm(const matrix &a, matrix &at);
///
long tranm(matrix &a);

///
long vxm(const vector &u, const matrix &a, vector &v);
///
long vxmt(const vector &u, const matrix &a, vector &v);

///
long mxv(const matrix &a, const vector &u, vector &v);
void mxv(const double *a, const double *b, double *c, long m, long n);
///
void mxvc(const double *a, const double *b, double *c, long m, long n);

///
long mtxv(const matrix &a, const vector &u, vector &v);
void mtxv(const double *a, const double *b, double *c, long m, long n);
///
void mtvc(const double *a, const double *b, double *c, long m, long n);

///
long vxv(const vector &u, const vector &v, matrix &a);
///
long vxmxv (const vector &v, const matrix &m, double &answer);
long vxmxv (const double *v, const double *m, long dim, double &answer);

long condense_vector(matrix &sm, vector &nf, ivector &cu);
///
long gause(const matrix &a, matrix &b,double zero);
///
long gause(const matrix &a, matrix &b, vector &r,double zero);
///
long gause(const matrix &a, matrix &b, vector &r, vector &sol,double zero);
///
long invm(const matrix &a, matrix &b,double zero);
///
long detm(const matrix &a, double &det);

///
long readm(XFILE *in, matrix &a);
///
long printm(const matrix &a, FILE *out = stdout, int prec = 3, int width = 11);

///
long mswapc(matrix &mat, long i, long j);
///
long mswapr(matrix &mat, long i, long j);

///
long condense_matrix(matrix &sm, ivector &cu);
///
long condense_vector(matrix &sm, vector &nf, ivector &cu);
///
void bdbj (double *k,const double *b,const double *d,double jac,long m,long n);
///
void bdbjac (matrix &d,const matrix &a,const matrix &b,const matrix &c,double jac);
///
void nnj (double *m,const double *n,double jac,long mm,long nn);
///
void nnjac (matrix &d,const matrix &a,const matrix &b,double jac);



void lgvectortransf (vector &g,const vector &l,const matrix &tmat);
void glvectortransf (const vector &g,vector &l,const matrix &tmat);
void glmatrixtransf (matrix &a,const matrix &tmat);
void lgmatrixtransf (matrix &a,const matrix &tmat);

void lgmatrixtransfblock (matrix &a, const matrix &t);
void glvectortransfblock (vector &v, const matrix &t);
void lgvectortransfblock (vector &v, const matrix &t);

///
void locglob (double *gv,const double *lv,const long *cn,long n);
void globloc (const double *gv,double *lv,const long *cn,long n);

///
void locvecmat(double *mat, const double *vect, const long *cn, long ci, long m, long n);

///
double det2d (const double *x,const double *y);
double det3d (const double *x,const double *y,const double *z);

///
void princ_val (matrix &v,vector &pval,matrix &pvect,long ni,double err,double zero,long n,long normalize);
///
void gemp (double *a,double *x,double *y,long n,long m,double limit,long pivot);
///
void lu_full (double *a,double *x,double *y,long n,double zero,long tc);
///
void mat_localize (matrix &gm,matrix &lm,long *rcn,long *ccn);
///
void matassem_lsm (double *lsm,vector &natcoord);
///
void rhsassem_lsm (double *rhs,vector &natcoord,vector &values);
///
void solve_lsm (double *lsm,double *lhs,double *rhs,double zero,long n,long m);

void extractm (matrix &a,matrix &b,long fi, long ncomp);

void extractblock (double *a,double *b,long n,long nr,long nc,long fri,long fci);

#endif
